@@ -1,7 +1,7 @@
 // synthv1.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -59,31 +59,47 @@ const float LFO_FREQ_MIN  = 0.4f;
 const float LFO_FREQ_MAX  = 40.0f;
 
 
+// hyperbolic-tangent fast approximation
+inline float synthv1_tanhf ( const float x )
+{
+	const float x2 = x * x;
+	return x * (27.0f + x2) / (27.0f + 9.0f * x2);
+}
+
+
 // sigmoids
 
 inline float synthv1_sigmoid ( const float x )
 {
 //	return 2.0f / (1.0f + ::expf(-5.0f * x)) - 1.0f;
-	return synthv1_fx_tanhf(2.0f * x);
+	return synthv1_tanhf(2.0f * x);
 }
 
-inline float synthv1_sigmoid0 ( const float x, const float t )
+inline float synthv1_sigmoid_0 ( const float x, const float t0 )
 {
-	const float t0 = t - 1.0f;
-	const float t1 = 1.0f - t;
-
-	return (x < t0 ? t0 : (x > t1 ? t1 : x * (1.5f - 0.5f * x * x)));
+	const float t1 = 1.0f - t0;
+#if 0
+	return (x < -1.0f ? -t1 : (x > +1.0f ? t1 : t1 * x * (1.5f - 0.5f * x * x)));
+#else
+	if (x > +t1)
+		return +t1 + t0 * synthv1_tanhf(+(x - t1) / t0);
+	else
+	if (x < -t1)
+		return -t1 - t0 * synthv1_tanhf(-(x + t1) / t0);
+	else
+		return x;
+#endif
 }
 
-inline float synthv1_sigmoid1 ( const float x, const float t = 0.001f )
+inline float synthv1_sigmoid_1 ( const float x, const float t0 = 0.01f )
 {
-	return 0.5f * (1.0f + synthv1_sigmoid0(2.0f * x - 1.0f, t));
+	return 0.5f * (1.0f + synthv1_sigmoid_0(2.0f * x - 1.0f, t0));
 }
 
 
 // velocity hard-split curve
 
-inline float synthv1_velocity ( float x, float p = 0.2f )
+inline float synthv1_velocity ( const float x, const float p = 0.2f )
 {
 	return ::powf(x, (1.0f - p));
 }
@@ -102,17 +118,23 @@ struct synthv1_env
 	struct State
 	{
 		// process
-		float value(uint32_t n) const
-			{ return level + float(n) * delta; }
-
-		float value2(uint32_t n) const
-			{ const float y = value(n); return y * y; }
+		float tick()
+		{
+			if (running && frames > 0) {
+				phase += delta;
+				value = c1 * phase * (2.0f - phase) + c0;
+				--frames;
+			}
+			return value;
+		}
 
 		// state
 		bool running;
 		Stage stage;
-		float level;
+		float phase;
 		float delta;
+		float value;
+		float c1, c0;
 		uint32_t frames;
 	};
 
@@ -120,37 +142,49 @@ struct synthv1_env
 	{
 		p->running = true;
 		p->stage = Attack;
-		p->level = 0.0f;
-		p->frames = int(*attack * *attack * max_frames);
-		if (p->frames > 0)
+		p->frames = uint32_t(*attack * *attack * max_frames);
+		p->phase = 0.0f;
+		if (p->frames > 0) {
 			p->delta = 1.0f / float(p->frames);
-		else
+			p->value = 0.0f;
+		} else {
 			p->delta = 0.0f;
+			p->value = 1.0f;
+		}
+		p->c1 = 1.0f;
+		p->c0 = 0.0f;
 	}
 
 	void next(State *p)
 	{
 		if (p->stage == Attack) {
 			p->stage = Decay;
-			p->level = 1.0f;
-			p->frames = int(*decay * *decay * max_frames);
+			p->frames = uint32_t(*decay * *decay * max_frames);
 			if (p->frames < min_frames) // prevent click on too fast decay
 				p->frames = min_frames;
-			p->delta = (*sustain - 1.0f) / float(p->frames);
+			p->phase = 0.0f;
+			p->delta = 1.0f / float(p->frames);
+			p->c1 = *sustain - 1.0f;
+			p->c0 = p->value;
 		}
 		else if (p->stage == Decay) {
 			p->running = false; // stay at this stage until note_off received
 			p->stage = Sustain;
-			p->level = *sustain;
 			p->frames = 0;
+			p->phase = 0.0f;
 			p->delta = 0.0f;
+			p->c1 = 0.0f;
+			p->c0 = p->value;
 		}
 		else if (p->stage == Release) {
 			p->running = false;
 			p->stage = Done;
-			p->level = 0.0f;
 			p->frames = 0;
+			p->phase = 0.0f;
 			p->delta = 0.0f;
+			p->value = 0.0f;
+			p->c1 = 0.0f;
+			p->c0 = 0.0f;
 		}
 	}
 
@@ -158,20 +192,26 @@ struct synthv1_env
 	{
 		p->running = true;
 		p->stage = Attack;
-		p->frames = int(*attack * *attack * max_frames);
+		p->frames = uint32_t(*attack * *attack * max_frames);
 		if (p->frames < min_frames) // prevent click on too fast attack
 			p->frames = min_frames;
-		p->delta = (1.0f - p->level) / float(p->frames);
+		p->phase = 0.0f;
+		p->delta = 1.0f / float(p->frames);
+		p->c1 = 1.0f - p->value;
+		p->c0 = p->value;
 	}
 
 	void note_off(State *p)
 	{
 		p->running = true;
 		p->stage = Release;
-		p->frames = int(*release * *release * max_frames);
+		p->frames = uint32_t(*release * *release * max_frames);
 		if (p->frames < min_frames) // prevent click on too fast release
 			p->frames = min_frames;
-		p->delta = -(p->level) / float(p->frames);
+		p->phase = 0.0f;
+		p->delta = 1.0f / float(p->frames);
+		p->c1 = -(p->value);
+		p->c0 = p->value;
 	}
 
 	void note_off_fast(State *p)
@@ -180,7 +220,10 @@ struct synthv1_env
 		p->stage = Release;
 		if (p->frames > min_frames)
 			p->frames = min_frames;
-		p->delta = -(p->level) / float(p->frames);
+		p->phase = 0.0f;
+		p->delta = 1.0f / float(p->frames);
+		p->c1 = -(p->value);
+		p->c0 = p->value;
 	}
 
 	// parameters
@@ -1351,6 +1394,11 @@ void synthv1_impl::allNotesOff (void)
 
 void synthv1_impl::reset (void)
 {
+//--legacy support < 0.3.0.4 -- begin
+	if (*m_del.bpm < 3.6f)
+		*m_del.bpm *= 100.0f;
+//--legacy support < 0.3.0.4 -- end.
+
 	m_vol1.reset(m_out1.volume, m_dca1.volume, &m_ctl.volume, &m_aux1.volume);
 	m_pan1.reset(m_out1.panning, &m_ctl.panning, &m_aux1.panning);
 	m_wid1.reset(m_out1.width);
@@ -1399,8 +1447,8 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 	// controls
 
-	const float lfo1_rate  = *m_lfo1.rate  * *m_lfo1.rate;
-	const float lfo2_rate  = *m_lfo2.rate  * *m_lfo2.rate;
+	const float lfo1_rate  = *m_lfo1.rate * *m_lfo1.rate;
+	const float lfo2_rate  = *m_lfo2.rate * *m_lfo2.rate;
 	const float lfo1_freq  = LFO_FREQ_MIN + lfo1_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
 	const float lfo2_freq  = LFO_FREQ_MIN + lfo2_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
 	const float lfo1_pitch = *m_lfo1.pitch * *m_lfo1.pitch;
@@ -1484,11 +1532,11 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 				// generators
 
-				const float lfo1_env = pv->lfo1_env.value2(j);
-				const float lfo2_env = pv->lfo2_env.value2(j);
+				const float lfo1_env = pv->lfo1_env.tick();
+				const float lfo2_env = pv->lfo2_env.tick();
 
-				const float lfo1  = pv->lfo1_sample * lfo1_env;
-				const float lfo2  = pv->lfo2_sample * lfo2_env;
+				const float lfo1 = pv->lfo1_sample * lfo1_env;
+				const float lfo2 = pv->lfo2_sample * lfo2_env;
 
 				float dco11 = pv->dco1_sample1 * pv->dco1_bal.value(j, 0);
 				float dco12 = pv->dco1_sample2 * pv->dco1_bal.value(j, 1);
@@ -1517,10 +1565,10 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				// filters
 
 				const float env1 = 0.5f * (1.0f + vel1
-					* *m_dcf1.envelope * pv->dcf1_env.value2(j));
-				const float cutoff1 = synthv1_sigmoid1(*m_dcf1.cutoff
+					* *m_dcf1.envelope * pv->dcf1_env.tick());
+				const float cutoff1 = synthv1_sigmoid_1(*m_dcf1.cutoff
 					* env1 * (1.0f + *m_lfo1.cutoff * lfo1));
-				const float reso1 = synthv1_sigmoid1(*m_dcf1.reso
+				const float reso1 = synthv1_sigmoid_1(*m_dcf1.reso
 					* env1 * (1.0f + *m_lfo1.reso * lfo1));
 
 				dco11 = pv->dcf11.output(dco11, cutoff1, reso1);
@@ -1531,10 +1579,10 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				}
 
 				const float env2 = 0.5f * (1.0f + vel2
-					* *m_dcf2.envelope * pv->dcf2_env.value2(j));
-				const float cutoff2 = synthv1_sigmoid1(*m_dcf2.cutoff
+					* *m_dcf2.envelope * pv->dcf2_env.tick());
+				const float cutoff2 = synthv1_sigmoid_1(*m_dcf2.cutoff
 					* env2 * (1.0f + *m_lfo2.cutoff * lfo2));
-				const float reso2 = synthv1_sigmoid1(*m_dcf2.reso
+				const float reso2 = synthv1_sigmoid_1(*m_dcf2.reso
 					* env2 * (1.0f + *m_lfo2.reso * lfo2));
 
 				dco21 = pv->dcf21.output(dco21, cutoff2, reso2);
@@ -1550,13 +1598,13 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				const float mid1 = 0.5f * (dco11 + dco12);
 				const float sid1 = 0.5f * (dco11 - dco12);
 				const float vol1 = vel1 * m_vol1.value(j)
-					* pv->dca1_env.value2(j);
+					* pv->dca1_env.tick();
 
 				const float wid2 = m_wid2.value(j);
 				const float mid2 = 0.5f * (dco21 + dco22);
 				const float sid2 = 0.5f * (dco21 - dco22);
 				const float vol2 = vel2 * m_vol2.value(j)
-					* pv->dca2_env.value2(j);
+					* pv->dca2_env.tick();
 
 				// outputs
 
@@ -1582,25 +1630,10 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 			// envelope countdowns
 
-			if (pv->dca1_env.running) {
-				if (pv->dca1_env.frames >= ngen) {
-					pv->dca1_env.frames -= ngen;
-					pv->dca1_env.level  += ngen * pv->dca1_env.delta;
-				}
-				else pv->dca1_env.frames = 0;
-				if (pv->dca1_env.frames == 0)
-					m_dca1.env.next(&pv->dca1_env);
-			}
-
-			if (pv->dca2_env.running) {
-				if (pv->dca2_env.frames >= ngen) {
-					pv->dca2_env.frames -= ngen;
-					pv->dca2_env.level  += ngen * pv->dca2_env.delta;
-				}
-				else pv->dca2_env.frames = 0;
-				if (pv->dca2_env.frames == 0)
-					m_dca2.env.next(&pv->dca2_env);
-			}
+			if (pv->dca1_env.running && pv->dca1_env.frames == 0)
+				m_dca1.env.next(&pv->dca1_env);
+			if (pv->dca2_env.running && pv->dca2_env.frames == 0)
+				m_dca2.env.next(&pv->dca2_env);
 
 			if (pv->dca1_env.stage == synthv1_env::Done &&
 				pv->dca2_env.stage == synthv1_env::Done) {
@@ -1609,42 +1642,14 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				free_voice(pv);
 				nblock = 0;
 			} else {
-				if (pv->dcf1_env.running) {
-					if (pv->dcf1_env.frames >= ngen) {
-						pv->dcf1_env.frames -= ngen;
-						pv->dcf1_env.level  += ngen * pv->dcf1_env.delta;
-					}
-					else pv->dcf1_env.frames = 0;
-					if (pv->dcf1_env.frames == 0)
-						m_dcf1.env.next(&pv->dcf1_env);
-				}
-				if (pv->dcf2_env.running) {
-					if (pv->dcf2_env.frames >= ngen) {
-						pv->dcf2_env.frames -= ngen;
-						pv->dcf2_env.level  += ngen * pv->dcf2_env.delta;
-					}
-					else pv->dcf2_env.frames = 0;
-					if (pv->dcf2_env.frames == 0)
-						m_dcf2.env.next(&pv->dcf2_env);
-				}
-				if (pv->lfo1_env.running) {
-					if (pv->lfo1_env.frames >= ngen) {
-						pv->lfo1_env.frames -= ngen;
-						pv->lfo1_env.level  += ngen * pv->lfo1_env.delta;
-					}
-					else pv->lfo1_env.frames = 0;
-					if (pv->lfo1_env.frames == 0)
-						m_lfo1.env.next(&pv->lfo1_env);
-				}
-				if (pv->lfo2_env.running) {
-					if (pv->lfo2_env.frames >= ngen) {
-						pv->lfo2_env.frames -= ngen;
-						pv->lfo2_env.level  += ngen * pv->lfo2_env.delta;
-					}
-					else pv->lfo2_env.frames = 0;
-					if (pv->lfo2_env.frames == 0)
-						m_lfo2.env.next(&pv->lfo2_env);
-				}
+				if (pv->dcf1_env.running && pv->dcf1_env.frames == 0)
+					m_dcf1.env.next(&pv->dcf1_env);
+				if (pv->dcf2_env.running && pv->dcf2_env.frames == 0)
+					m_dcf2.env.next(&pv->dcf2_env);
+				if (pv->lfo1_env.running && pv->lfo1_env.frames == 0)
+					m_lfo1.env.next(&pv->lfo1_env);
+				if (pv->lfo2_env.running && pv->lfo2_env.frames == 0)
+					m_lfo2.env.next(&pv->lfo2_env);
 			}
 		}
 
@@ -1671,7 +1676,7 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 			*m_pha.rate, *m_pha.feedb, *m_pha.depth, *m_pha.daft * float(k));
 		// delay
 		m_delay[k].process(in, nframes, *m_del.wet,
-			*m_del.delay, *m_del.feedb, *m_del.bpm * 100.0f);
+			*m_del.delay, *m_del.feedb, *m_del.bpm);
 		// compressor
 		if (int(*m_dyn.compress) > 0)
 			m_comp[k].process(in, nframes);
