@@ -1,7 +1,7 @@
 // synthv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -149,7 +149,7 @@ struct {
 	{ "DEL1_WET",       0.0f },
 	{ "DEL1_DELAY",     0.5f },
 	{ "DEL1_FEEDB",     0.5f },
-	{ "DEL1_BPM",       1.8f },
+	{ "DEL1_BPM",     180.0f },
 	{ "DYN1_COMPRESS",  0.0f },
 	{ "DYN1_LIMIT",     1.0f }
 };
@@ -303,8 +303,9 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Out2PanningKnob->setMaximum(+1.0f);
 
 	// Effects (delay BPM)
-	m_ui.Del1BpmKnob->setMinimum(0.0f);
-	m_ui.Del1BpmKnob->setMaximum(3.6f);
+	m_ui.Del1BpmKnob->setScale(1.0f);
+	m_ui.Del1BpmKnob->setMinimum(3.6f);
+	m_ui.Del1BpmKnob->setMaximum(360.0f);
 
 
 	// DCO1
@@ -1001,6 +1002,10 @@ void synthv1widget::loadPreset ( const QString& sFilename )
 							if (!sName.isEmpty() && s_hash.contains(sName))
 								index = s_hash.value(sName);
 							float fValue = eParam.text().toFloat();
+						//--legacy support < 0.3.0.4 -- begin
+							if (index == synthv1::DEL1_BPM && fValue < 3.6f)
+								fValue *= 100.0f;
+						//--legacy support < 0.3.0.4 -- end.
 							setParamValue(index, fValue);
 							updateParam(index, fValue);
 							m_params_ab[index] = fValue;
